# 公主（举）包
from bs4 import BeautifulSoup


def set_attr(options: dict, attr: list, value: any) -> str:
    i = 0
    for x in attr:
        if i == 0:
            exec_str = "options['%s']" % x
            i += 1
        else:
            exec_str += "['%s']" % x

    exec_str += " = %s" % value
    print(exec_str)
    exec(exec_str)
    # print(options)
    return exec_str


def load_html(file_path=None):
    html_temp = '<!DOCTYPE html>\
<html>\
\
<head>\
    <meta charset="utf-8" />\
    <meta http-equiv="X-UA-Compatible" content="IE=edge">\
    <meta name="viewport" content="width=device-width, initial-scale=1">\
    <title></title>\
\
    <!-- ZUI Javascript 依赖 jQuery -->\
    <script src="https://cdn.staticfile.org/echarts/4.3.0/echarts.min.js"></script>\
</head>\
\
<body>\
    <div class="container"></div>\
</body>\
<script src="js/charts_cfg.js"></script>\
\
</html>'
    if file_path == None:
        return BeautifulSoup(html_temp, features="lxml")
    else:
        return BeautifulSoup(open(file_path),features="lxml")

def load_data(data) -> str:
    return str(data)


def load_data_from_file(path: str, args_name: list, split=","):
    pass


def js_init_echart(chart):
    global_args = chart.get_global_args()
    options = chart.get_options()
    i = 0
    for arg in global_args["init"]:
        if i == 0:
            init_js = "var %s = echarts.init(\n\
                %s=%s,   \n \
            " % (
                global_args["id"],
                arg,
                global_args["init"][arg] ,
            )
            i = 1
        else:

            init_js += "%s = %s,\n" % (
                arg,
                global_args["init"][arg]if global_args["init"][arg] != None else "null",
            )
    init_js += ");\n"
    print(init_js)
    return init_js


def js_init_chartOption(chart):
    global_args = chart.get_global_args()
    options = chart.get_options()
    options_js = "%s.setOption(%s)" % (
        global_args["id"],
        options
    )
    options_js +=";\n"
    return options_js


def echart2html(chart,log_path=None):
    # echart_container = '&lt;div id="%s" style="float: left;"></div>' % chart.get_global_args()["id"]
    if log_path == None:
        soup = load_html()
    else:
        soup = load_html(file_path=log_path+"/html/main.html")
    echart_container = soup.new_tag("div", id=chart.get_global_args()["id"],style="float: left;")
    soup.div.append(echart_container)
    return soup.prettify()

# soup = load_html().prettify
# print(soup)




from abc import abstractclassmethod, abstractmethod
from science_utils_k.utils import princess


class EChart(object):
    def __init__(self, global_args=None, options=None):
        if global_args == None:
            self.global_args = {
                "id": "chart",
                "init": {
                    "dom": "document.getElementById('chart')",
                    "theme": None,
                    'options': {
                        "width": 600,
                        "height": 500
                    }

                }
            }
        else:
            self.global_args = global_args
    def get_global_args(self):
        return self.global_args

    def set_global_args(self, attrs, values):
        for attr, value in zip(attrs, values):
            # print(attr)
            # print(value)
            attr = attr.split(".")
            princess.set_attr(self.global_args, attr, value)
    def output(type):
        pass



from re import S
from tracemalloc import start
from science_utils_k.echarts.EChart import EChart
from science_utils_k.utils import princess
import time
import os

class LineChart(EChart):

    def __init__(self, global_args=None, options=None):
        super(LineChart, self).__init__(global_args)
        if options == None:
            self.options = {
                'title': {
                    'text': 'Classification',
                },
                'tooltip': {},
                'legend': {},
                'xAxis': {
                    'name': "epoch",
                    'nameLocation': "end",
                    'data': None,
                    'axisLabel': {
                        'show': 'true'
                    }
                },
                'yAxis': {},
                'series': None
            }
        else:
            self.options = options

    def get_options(self):
        return self.options

    def set_options(self, attrs, values):
        for attr, value in zip(attrs, values):
            # print(attr)
            # print(value)
            attr = attr.split(".")
            princess.set_attr(self.options, attr, value)

    def output(self, time_stamp,type="html", log_root_path="log"):
        
        o_path = log_root_path + "/"+time_stamp
        if type == "html":
            if not os.path.exists("log"):
                os.makedirs("log")
            if time_stamp not in os.listdir("log"):
                os.makedirs(o_path)

            if "html" not in os.listdir(o_path):
                os.makedirs(o_path+"/html")
            if "js" not in os.listdir(o_path+"/html"):
                os.makedirs(o_path+"/html/js")
            # os.makedirs(o_path+"/html/js/")

            # 如果main.html已存在，从以有文件读入html。否则使用默认html模板
            if "main.html" in os.listdir(o_path+"/html"):
                echart_html = princess.echart2html(self,log_path=o_path)
            else:
                echart_html = princess.echart2html(self)
            
            # write html file
            fd = open(o_path+"/html/main.html", "w")
            fd.write(str(echart_html))
            fd.close()

            # write js file
            fd = open(o_path+"/html/js/charts_cfg.js","a")
            fd.write(princess.js_init_echart(self))
            fd.write(princess.js_init_chartOption(self))
            fd.close()



import os
import time
from turtle import fd


class Logger(object):
    #time = time.strftime("%Y-%m-%d %H-%M-%S", time.localtime()) # 创建时间
    @staticmethod
    def log2terminal(*s, tofile=False,log_root_path="log",time_stamp=None):
        str_content=""
        for x in s:
            str_content += str(x) 
        current_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()) 
        print(current_time,":",str_content)
        if tofile:
            Logger.log2file(s,time_stamp=time_stamp,log_root_path=log_root_path)
    
    @staticmethod
    def log2file(*s,time_stamp, log_root_path="log"):
        str_content=""

        # 拼接内容
        for x in s:
            str_content += str(x)

        if not os.path.exists("log"):
            os.mkdir("log")
        if time_stamp not in os.listdir("log"):
            os.makedirs(log_root_path+"/"+time_stamp)

        current_time = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime()) 
        log_content = current_time+": "+str_content+"\n"

        # write
        fd = open(log_root_path+"/"+time_stamp+"/log.txt", "a")
        fd.write(log_content)
        fd.close()

    @staticmethod
    def save_hyperparameter(hyperparameter, filepath='log/hyperparameter.txt'):
        fd = open(filepath, "w")

        # write
        for key in hyperparameter.keys(): 
            parameter = "%s : %s\n" %(key, str(hyperparameter[key]))
            fd.write(parameter)
        fd.close()


